$! BUILDKIT.COM - Creates the complete installation kit with multiple
$! platforms and multiple network stacks support. The procedure must be run
$! from the root directory where PINE sources are, the same where you must
$! execute VMSBUILD for complete package compilation.
$!
$! options are:
$!
$! P1 (compiling run):
$!    null	- Compile the package on current (VAX or AXP) platform, using 
$!		  default options NOOPTIMIZE and best available compiler
$!    OPTIMIZE	- Forces OPTIMIZATION on compile stage
$!    VAXC	- Force use of VAXC
$!    GNUC	- Force use of GNUC
$!
$! P1 (packing run):
$!    PACKBCK	- Pack installation kit into backup save set specified by P2
$!    PACKZIP	- Pack installation kit into zipped save set specified by P2
$!    NETLIB	- Pack NETLIB support only into installation kit
$!		  (used jointly with PACKBCK or PACKZIP only)
$!    MULTINET	- Pack MULTINET support only into installation kit
$!		  (used jointly with PACKBCK or PACKZIP only)
$!    NOTCP	- Pack no TCP/IP stack support only into installation kit
$!		  (used jointly with PACKBCK or PACKZIP only)
$!
$! NOTE: if nor NETLIB nor MULTINET nor NOTCP parameters are specified,
$!       then a full stacks support package is prepared.
$!
$! P2:
$!    kit-name	- name of save set when installation kit is stored 
$!		  (default: PINEVMSBINv1v2.BCK or PINEVMSBINv1v2.ZIP). 
$!		  Ignored if P1 is not "PACKBCK" or "PACKZIP"
$!
$! NOTE: BUILDKIT must be run three times, on VAX and AXP platform, on hosts
$!       where both MULTINET and NETLIB are installed to build the complete
$!       kit. The full sequence is
$!
$!       on a VAX host:
$!
$!          $ @BUILDKIT
$!
$!       on an AXP host:
$!
$!          $ @BUILDKIT
$!          $ @BUILDKIT PACKBCK PINEVMSBIN0391A.BCK
$!
$!  Claudio Allocchio, Sincrotrone Trieste, 05 Feb 1997
$!
$! Define here the PINE version and subversion:
$!
$ v1 = "03"
$ v2 = "91A"
$!
$ OBJECTS = "[...]*.OBJ_*"
$ if f$locate("NOTCP",P1) .lt. f$length(P1) then OBJECTS = "[...]*.OBJ_*NON"
$ if f$locate("MULTINET",P1) .lt. f$length(P1) then OBJECTS = "[...]*.OBJ_*MUL"
$ if f$locate("NETLIB",P1) .lt. f$length(P1) then OBJECTS = "[...]*.OBJ_*NET"
$ if f$locate("PACKBCK",P1) .lt. f$length(P1) then goto PACK_stage_b
$ if f$locate("PACKZIP",P1) .lt. f$length(P1) then goto PACK_stage_z
$!
$ VAX = (f$getsyi("HW_MODEL") .le. 1023)
$ if f$search("[...]*.obj;*") .nes. "" then delete/nolog/noconf [...]*.obj;*
$ if f$search("[...]*.olb;*") .nes. "" then delete/nolog/noconf [...]*.olb;*
$!
$ if P1 .eqs. ""
$   then @VMSBUILD NETLIB,COMPILE-ONLY
$   else @VMSBUILD 'P1',NETLIB,COMPILE-ONLY
$ endif
$ if VAX
$   then rename/nolog/noconf [...]*.obj [...]*.obj_VAXNET
$   else rename/nolog/noconf [...]*.obj [...]*.obj_AXPNET
$ endif
$!
$ if P1 .eqs. ""
$   then @VMSBUILD MULTINET,COMPILE-ONLY
$   else @VMSBUILD 'P1',MULTINET,COMPILE-ONLY
$ endif
$ if VAX
$   then rename/nolog/noconf [...]*.obj [...]*.obj_VAXMUL
$   else rename/nolog/noconf [...]*.obj [...]*.obj_AXPMUL
$ endif
$!
$ if P1 .eqs. ""
$   then @VMSBUILD NOTCP,COMPILE-ONLY
$   else @VMSBUILD 'P1',NOTCP,COMPILE-ONLY
$ endif
$ if VAX
$   then rename/nolog/noconf [...]*.obj [...]*.obj_VAXNON
$        write sys$output "%BUILDKIT-I-COMPDONE, kit for VAX platform compiled"
$   else rename/nolog/noconf [...]*.obj [...]*.obj_AXPNON
$        write sys$output "%BUILDKIT-I-COMPDONE, kit for AXP platform compiled"
$ endif
$!
$ if f$search("[...]*.olb;*") .nes. "" then delete/nolog/noconf [...]*.olb;*
$ exit
$!
$ PACK_stage_b:
$ if P2 .eqs. "" then P2 = "PINEVMSBIN''v1'''v2'.BCK"
$ if f$locate("NOTCP",P1) .lt. f$length(P1) then P2 = "NONET-"+P2
$ if f$locate("MULTINET",P1) .lt. f$length(P1) then P2 = "MULTINET-"+P2
$ if f$locate("NETLIB",P1) .lt. f$length(P1) then P2 = "NETLIB-"+P2
$ write sys$output "%BUILDKIT-I-PACK, start packing..."
$ backup/log AAAREADME.TXT,00BINARIES.TXT,MAIL.MAI-TEMPLATE,[.DOC]*.*,[...]*.COM,[...]*.OPT,'OBJECTS' -
             'P2'/SAVE
$ if f$search("[...]*.olb;*") .nes. "" then delete/nolog/noconf [...]*.olb;*
$ write sys$output " "
$ write sys$output "%BUILDKIT-I-PACK, installation kit: ",P2
$ exit
$!
$ PACK_stage_z:
$ if P2 .eqs. "" then P2 = "PINEVMSBIN''v1'''v2'.ZIP"
$ if f$locate("NOTCP",P1) .lt. f$length(P1) then P2 = "NONET-"+P2
$ if f$locate("MULTINET",P1) .lt. f$length(P1) then P2 = "MULTINET-"+P2
$ if f$locate("NETLIB",P1) .lt. f$length(P1) then P2 = "NETLIB-"+P2
$ write sys$output "%BUILDKIT-I-PACK, start packing..."
$ zip "-V" -wvr 'P2' AAAREADME.TXT 00BINARIES.TXT MAIL.MAI-TEMPLATE [.DOC]*.* [...]*.COM [...]*.OPT 'OBJECTS'
$ if f$search("[...]*.olb;*") .nes. "" then delete/nolog/noconf [...]*.olb;*
$ write sys$output " "
$ write sys$output "%BUILDKIT-I-PACK, installation kit: ",P2
$ exit
