$! Purpose:
$!	Set up logical names to control the actions of the C-CLIENT library
$!	Unless otherwise stated, all of these are optional.
$!
$! Author:
$!	1.0	Andy Harper	March 1997	Original Version
$!	1.1	Andy Harper	May 1997	Add missing definition of DEFSYS
$!
$!
$ defsys = "define /system /exec"
$
$! -----------------
$! MAIL.MAI prototyping - this logical must be defined to a valid MAIL.MAI file
$!		so that the C-CLIENT CREATE function can work. The file should
$!		be empty of folders.
$!
$! -----------------
$ defsys	CCLIENT_MAILPROTOTYPE	imapd_conf_dir:mail.mai
$
$
$! -----------------
$! C-Client Logging -- define this logical to get additional logging
$!		information written to the applications debug log
$!		Range is 0 - 25 currently, the higher the value, the more
$!		info will be written.
$!
$!		Note: affects only C-CLIENT modules built with the LOGTRACE
$!		option.
$! -----------------
$! defsys		cclient_loglevel 25
$
$
$
$! -----------------
$! INBOX folder name - if this logical is defined to the name of a mail folder
$!		then, when INBOX is selected as the folder, this folder in the
$!		default mail file will be selected. If not defined, then
$!		NEWMAIL will be selected (which is the standard VMS MAIL inbox).
$!
$! -----------------
$! defsys	CCLIENT_INBOXFOLDERNAME	"ALTNEWMAIL"
$
$
$! -----------------
$! INBOX filing - if this logical is defined to the name of a mail folder,
$!		then, when INBOX is closed, all SEEN (but not DELETED) messages
$!		will be moved to the named folder. For compatibility with VMS
$!		MAIL, define this to the string "MAIL"; for compatibility with
$!		most other IMAP servers, leave it undefined so that messages
$!		are left where they are.
$!
$! -----------------
$! defsys CCLIENT_AUTOFILEINBOX "MAIL"
$!
$!
$! -----------------
$! Mailbox Purge/Reclaim -- if this logical is defined, the mailbox is purged
$!			 -- and the deleted bytes reclaimed each time a folder
$!			 -- is closed. Otherwise, purging and reclaiming is not
$!			 -- performed.
$! -----------------
$! defsys CCLIENT_MAILFILE_PURGE_RECLAIM "X"	! actual value unimportant
$!
$!
$! -----------------
$! RFC1405 format control -- Specify how non-mapped local DECnet addresses are
$!                        -- rewritten.
$!	By default, DECnet address formats are converted into RFC822 format
$!	according to the RFC1405 standard for local addresses. Defining this
$!	logical name (to anything) causes an older behaviour: mapping of
$!	"[namespace:.]node::user" results int "user@node"
$!
$!	This logical name is ignored if there is a mapping specified by logical
$!	names for the particular nodename/namespace combination. See below.
$!
$! -----------------
$! defsys		CCLIENT_norfc1405	" "
$!
$!
$! -----------------
$! DECnet node mappings -- THESE ARE EXAMPLES ONLY. Define for your local nodes
$!
$!	There are several possibilities for a DECnet address and various
$!	logical may be defined to deal with them. The default case is to encode
$!	the DECnet address into the RFC1405 preferred format.
$!
$!	* LOCAL NODES THAT HAVE NO DECNET:
$!		* Address format:	USERNAME
$!		* Default mapping	USERNAME@DOMAIN or USERNAME
$!		* Non-RFC1405 default:  USERNAME@DOMAIN or USERNAME
$!		* Logical name:		CCLIENT_NODE
$!		* Example:
$!		   $ define /system CCLIENT_NODE "cityburg.edu"
$!		* Resultant Mapping:
$!		   USERNAME@cityburg.edu
$!
$!	* DECNET PHASE IV NODES:
$!		* Address format:	NODE::USERNAME
$!		* Default mapping:	"NODE::USERNAME"@DOMAIN
$!		* Non-RFC1405 default:  USERNAME@NODE
$!		* Logical name:		CCLIENT_NODE_'nodename'
$!		* Example:
$!		   $ define /system CCLIENT_NODE_BROWN "brown.cityburg.edu"
$!		* Resultant Mapping:
$!		   USERNAME@brown.cityburg.edu
$!
$!	* DECNET/OSI (Phase V) NODES:
$!		* Address format:	NAMESPACE:.NODE::USERNAME
$!		* Default mapping:	"NAMESPACE:.NODE::USERNAME"@DOMAIN
$!		* Non-RFC1405 default:	USERNAME@NODE
$!		* Logical name:		CCLIENT_NODE_'namespace'_'node'
$!		* Example:
$!		   $ define /system CCLIENT_NODE_LOCAL_GREY "grey.cityburg.edu"
$!		* Resultant Mapping:
$!		   USERNAME@grey.cityburg.edu
$!
$!	Where no logical name can be found to provide the mapping, the DOMAIN
$!	name used is derived from the current host name. This can be configured
$!	to a different domain using the logical name CCLIENT_MAIL11_GW (see
$!	below).
$!
$!	Note that care should be taken to ensure that the resulting DOMAIN is
$!	the same as that specified in PINE.CONF or PINE.CONF-FIXED file for that
$!	node (the 'user-domain' variable). This ensures consistency of address
$!	presentation.
$! -----------------
$! defsys		cclient_node_local_bay		"KCL.AC.UK"
$! defsys		cclient_node_local_alder	"KCL.AC.UK"
$! defsys		cclient_node_local_grove	"KCL.AC.UK"
$! defsys		cclient_node_local_hazel	"HAZEL.CC.KCL.AC.UK"
$! defsys		cclient_node_local_phclus	"MAXWELL.PH.KCL.AC.UK"
$! defsys		cclient_node_local_basher 	"KCL.AC.UK"
$!
$! defsys		cclient_node_bay		"KCL.AC.UK"
$! defsys		cclient_node_alder 		"KCL.AC.UK"
$! defsys		cclient_node_grove		"KCL.AC.UK"
$! defsys		cclient_node_hazel		"HAZEL.CC.KCL.AC.UK"
$! defsys		cclient_node_phclus		"MAXWELL.PH.KCL.AC.UK"
$! defsys		cclient_node_basher		"KCL.AC.UK"
$!
$! defsys		CCLIENT_NODE			"KCL.AC.UK"
$!
$!
$! -----------------
$! DECNET Mail11 gateway - Define the name of the gateway machine that can
$!		handle inbound SMTP to DECNET mail routing for an address of
$!		this form:
$!			"NODE::USER"@DOMAIN
$!			"NAMESPACE::.NODE::USER"@DOMAIN
$!
$!		DECNET FROM: addresses not mapped by any of the above formats
$!		will then be mapped into this form. 
$!
$! -----------------
$! defsys		CCLIENT_GW_MAIL11		"ALDER.CC.KCL.AC.UK"
$!
$!
$! -----------------
$! X.400 Gateway -- Specify the name of an X.400 gateway to add to rewritten
$!		X.400 addresses. Addresses of the form:
$!			MRGATE:"c=gb::a= ::o=org::g=fred"
$!		are rewritten as:
$!			"/c=gb/a= /o=org/g=fred/"@X400-GW
$!
$!		This logical name defines the relay host written instead of the
$!		string "X400-GW"
$!
$! -----------------
$!  defsys		CCLIENT_GW_X400		"x400-relay.ja.net"
$!
$!
$! -----------------
$! RFC822 Header Position -- Allows RFC822 headers to appear at the end
$!                           of the message body.
$!
$!	Normally, the RFC822 mail headers appear at the beginning of the
$!	message. Some mailers can be configured to place them at the end to
$!	make it easier to read messages without having to skip over the headers
$!	first. The headers are separated from the body by a special line.
$!
$!	This logical name specifies the special separator line. If defined, the
$!	message parser will scan for this line in the message body and treat
$!	everything following it as the headers for the message. If the special
$!	line is not found in the message body, the rewritten VMS MAIL headers
$!	will be returned instead and any (apparent) RFC822 headers at the start
$!	of the message body will be treated as text.
$! -----------------
$! defsys		CCLIENT_HEADERS_ATEND	"----- RFC822 HEADERS ---"
