$! Parameters:
$!	P1	Build options
$!			NOTCP, MULTINET, NETLIB	- Select TCP stack support
$!			COMPILE-ONLY		- Bypass link stage
$!			LINK-ONLY		- Bypass compilation stage
$!			VAXC			- Force use of VAXC
$!			GNUC			- Force use of GNUC
$!			DEBUG			- compile/link with debugger support
$!			NOTRACEBACK		- links without traceback
$!			OPTIMIZE		- Forces optimization if DEBUG not set
$!
$!	P2	List of modules to be compiled
$!			a,b,c,...		- Compile a,b,c,... only
$!			NULL			- Compile everything
$!
$! Author:
$!	Andy Harper, Kings College London
$!
$
$! -------------------------------------------------------------------------
$! Define location for object files and executables
$! -------------------------------------------------------------------------
$ Arch = "VAX"
$ if f$getsyi("VERSION") .ges. "V5.5" then $ Arch = f$getsyi("ARCH_NAME")
$ OBJDIR      = "[-.OBJ_''Arch']"
$ EXEDIR      = "[-.EXE_''Arch']"
$ if f$parse(OBJDIR) .eqs. "" then $ create/dir 'OBJDIR'
$ if f$parse(EXEDIR) .eqs. "" then $ create/dir 'EXEDIR'
$ CCLOBJDIR   = "[-.OBJ_''Arch'.C-CLIENT]
$
$ CCLIENT_DIR = "[-.IMAP.ANSI.C-CLIENT]"
$ Includes    ="[],''CCLIENT_DIR'"
$
$! -------------------------------------------------------------------------
$! Check compile and link conflicts
$! -------------------------------------------------------------------------
$ COMPILEONLY = f$locate("COMPILE-ONLY",P1) .lt. f$length(P1)
$ LINKONLY    = f$locate("LINK-ONLY",   P1) .lt. f$length(P1)
$ if (COMPILEONLY .and. LINKONLY)
$   then
$     write sys$output "%VMSBUILD-E-CONFLICT, Conflicting compile/link options"
$     exit
$ endif
$!
$ if .not. LINKONLY
$   then
$! ----------------------------------------------------------------------------
$! Create the VMS specific modules
$! ----------------------------------------------------------------------------
$     OPEN/WRITE OUTFILE DATE.C
$     WRITE OUTFILE "char datestamp[]=""''F$TIME()'"";"
$     close outfile
$     purge /nolog/noconf DATE.C
$     COPY/nolog/noconf OS_VMS.H OS.H;
$     purge/nolog/noconf os.h
$     COPY/nolog/noconf HELPTEXT_VMS.H HELPTEXT.H;
$     purge/nolog/noconf helptext.h
$     COPY/nolog/noconf HELPTEXT_VMS.C HELPTEXT.C;
$     purge/nolog/noconf helptext.c
$     COPY/nolog/noconf INIT_VMS.C INIT.C;
$     purge/nolog/noconf init.c
$ endif
$
$! ----------------------------------------------------------------------------
$! Initialize compile and link options
$! ----------------------------------------------------------------------------
$ CC_PREF = ""
$ LINK_OPT = ""
$ sysdef = ""
$ netdef = ""
$ arpdef = ""
$
$ if .not. LINKONLY then goto check_compiler
$!
$! check we have the object files available
$!
$ if F$SEARCH("''OBJDIR'PINE.OBJ") .eqs. ""
$   then
$     write sys$output "%VMSBUILD-E-NOOBJ, No object files detected"
$     exit
$ endif
$ write sys$output "%VMSBUILD-I-CHKCOMPILER, checking for used compiler"
$ if f$search("PINE.ANALYZE") .nes. "" then -
  delete/nolog/noconf PINE.ANALYZE;*
$ if f$search("PINE.COMPILER") .nes. "" then -
  delete/nolog/noconf PINE.COMPILER;*
$ analize/object/output=PINE.ANALYZE 'OBJDIR'PINE.OBJ/MHD
$ if .not. $status
$   then
$     write sys$output "%VMSBUILD-E-BADOBJ, BAD object files detected"
$     exit
$ endif
$ search PINE.ANALYZE "LANGUAGE PROCESSOR HEADER"/wind=(0,3) -
         /out=PINE.COMPILER
$ if f$search("PINE.ANALYZE") .nes. "" then -
  delete/nolog/noconf PINE.ANALYZE;*
$ COMPILER = "GNU C"
$ search PINE.COMPILER "VAX C"/out=NL:
$ if $status .eq. 1 then $ COMPILER = "VAX C"
$ search PINE.COMPILER "DEC C"/out=NL:
$ if $status .eq. 1 then $ COMPILER = "DEC C"
$ if f$search("PINE.COMPILER") .nes. "" then -
  delete/nolog/noconf PINE.COMPILER;*
$ HAVEDECC = 0
$ HAVEVAXC = 0
$ HAVEGNUC = 0
$ write sys$output "%VMSBUILD-I-COMPILER, source was compiled with ",COMPILER
$ goto compiler_done
$!
$ check_compiler:
$! ----------------------------------------------------------------------------
$! Select compiler; DEC C, VAX C and GNU C supported
$! ----------------------------------------------------------------------------
$! Find out which compilers we have
$ HAVEVAXC = f$search("sys$system:vaxc.exe") .nes. ""
$ HAVEDECC = f$search("sys$system:decc$compiler.exe") .nes. ""
$ HAVEGNUC = F$SEARCH("GNU_CC:[000000]GCC.EXE")       .NES. ""
$
$! ... and which one to use in preference (if any)
$ FORCEVAXC= f$locate("VAXC",P1) .lt. f$length(P1)
$ FORCEGNUC = f$locate("GNUC",P1) .lt. f$length(P1)
$ if .not. (HAVEDECC .or. HAVEVAXC .or. HAVEGNUC)
$   then
$     write sys$output "%VMSBUILD-E-NOCC, No supported C compiler detected"
$     exit
$ endif
$
$
$! Auto-select, in order of preference, the best compiler available
$ COMPILER = ""
$ if (HAVEGNUC) then $ COMPILER = "GNU C"
$ if (HAVEVAXC) then $ COMPILER = "VAX C"
$ if (HAVEDECC) then $ COMPILER = "DEC C"
$
$! Multiple compilers specified?
$ if (FORCEVAXC .and. FORCEGNUC)
$   then
$     write sys$output "%VMSBUILD-E-AMBIG, Multiple C compilers specified"
$     exit
$ endif
$
$! Has user requested GNU C?
$ if FORCEGNUC
$   then
$     if .not. HAVEGNUC
$       then
$         write sys$output "%VMSBUILD-E-NOGNUC, GNUC selected but not installed"
$         exit
$     endif
$     COMPILER = "GNU C"
$ endif
$
$! Has user requested VAX C
$ if (FORCEVAXC)
$   then
$     if .not. HAVEVAXC
$       then
$         write sys$output "%VMSBUILD-E-NOVAXC, VAXC selected but not installed"
$         exit
$     endif
$     COMPILER = "VAX C"
$ endif
$!
$ compiler_done:
!
$ IF (COMPILER .eqs. "GNU C")
$   THEN
$       if f$type(GCC) .nes. ""
$         then
$           CC = GCC		! Defined foreign
$         else
$           CC = "GCC"		! Defined via SET COMMAND
$       endif
$
$       LINK_OPT = ",[-]VMS_GCC_LINK/OPTION"
$ ENDIF
$
$ IF (COMPILER .eqs. "VAX C")
$   THEN
$       if (HAVEDECC) then $ CC_PREF = CC_PREF + "/VAXC"
$       
$       sysdef   = "SYS$LIBRARY"
$	netdef   = "SYS$LIBRARY"
$	LINK_OPT = ",[-]VMS_VAXC_LINK/OPTION"
$ ENDIF
$
$ IF (COMPILER .eqs. "DEC C")
$   THEN
$       if (HAVEVAXC) then $ CC_PREF = CC_PREF + "/DECC"
$
$	CC_PREF = CC_PREF + "/STANDARD=VAXC"
$	LINK_OPT = ""
$ ENDIF
$
$! Deassign any prior logicals in case we previously run something
$ IF F$TRNLNM("SYS")     .NES. "" THEN $ DEASSIGN SYS
$ IF F$TRNLNM("NETINET") .NES. "" THEN $ DEASSIGN NETINET
$ IF F$TRNLNM("ARPA")    .NES. "" THEN $ DEASSIGN ARPA
$
$! ----------------------------------------------------------------------------
$! Set up logical names pointing to system libraries
$! ----------------------------------------------------------------------------
$ if (sysdef .nes. "") then $ define /nolog sys     'sysdef'
$ if (netdef .nes. "") then $ define /nolog netinet 'netdef'
$ if (arpdef .nes. "") then $ define /nolog arpa    'arpdef'
$ 
$! ----------------------------------------------------------------------------
$! Select network stack;  NETLIB, MULTINET, NOTCP supported
$! ----------------------------------------------------------------------------
$! Find out which stacks are available
$ HAVENETLIB    = f$trnlnm("NETLIB_DIR","LNM$SYSTEM") .nes. ""
$ HAVEMULTINET  = f$trnlnm("MULTINET","LNM$SYSTEM") .nes. ""
$
$! ... Select the one to use
$ FORCENETLIB   = f$integer(f$locate("NETLIB",P1)   .lt. f$length(P1))
$ FORCEMULTINET = f$integer(f$locate("MULTINET",P1) .lt. f$length(P1))
$ FORCENOTCP    = f$integer(f$locate("NOTCP",P1)    .lt. f$length(P1))
$ if ( (FORCEMULTINET + FORCENETLIB + FORCENOTCP) .gt. 1 )
$   then
$     write sys$output "%VMSBUILD-E-CONFLTCP, Conflicting TCP stacks specified"
$     exit
$ endif
$
$! Pick best stack available
$ NETSTACK = "NO"
$ if (HAVEMULTINET) then $ NETSTACK = "MULTINET"
$ if (HAVENETLIB)   then $ NETSTACK = "NETLIB"
$
$ if (FORCENOTCP)
$   then
$     NETSTACK = "NO"
$ endif
$
$ if (FORCEMULTINET)
$   then
$     if .not. HAVEMULTINET
$       then
$         write sys$output "%VMSBUILD-E-NOTCP, MULTINET tcp not installed"
$         exit
$     endif
$     NETSTACK = "MULTINET"
$ endif
$
$ if (FORCENETLIB)
$   then
$     if .not. HAVENETLIB
$       then
$         write sys$output "%VMSBUILD-E-NOTCP, NETLIB tcp not installed"
$         exit
$     endif
$     NETSTACK = "NETLIB"
$ endif
$
$ LINK_OPT = ",[-]VMS_''NETSTACK'_LINK/OPTION" + LINK_OPT
$
$! ----------------------------------------------------------------------------
$! Set options dependent on debugging requirements
$! ----------------------------------------------------------------------------
$ IF F$LOCATE("DEBUG",P1) .lt. f$length(p1)
$   THEN			! Build a debug version of PINE
$     CC_PREF = CC_PREF + "/DEBUG /NOOPTIMIZE"
$     LINK   = "LINK /DEBUG"
$   ELSE
$     IF F$LOCATE("OPTIMIZE",P1) .lt. f$length(P1)	! /OPT is not default!
$       THEN
$         CC_PREF = CC_PREF + "/OPTIMIZE"
$       ELSE
$         CC_PREF = CC_PREF + "/NOOPTIMIZE"
$     ENDIF
$     LINK   = "LINK"
$     IF F$LOCATE("NOTRACEBACK",P1) .lt. f$length(P1)	! /NOTRACEBACK ??
$       THEN
$          LINK_OPT = LINK_OPT + "/NOTRACE"
$     ENDIF
$ ENDIF
$
$ if COMPILEONLY then goto just_compile
$ if LINKONLY then $ goto LINK
$!
$! ----------------------------------------------------------------------------
$! Build software
$! ----------------------------------------------------------------------------
$ write sys$output "%VMSBUILD-I-BUILD, Building PINE using ", COMPILER, " with ", NETSTACK, " TCP/IP support"
$ just_compile:
$! CC_PREF = CC_PREF + "/DEFINE=(SIGUSR2=SIGKILL,DEBUG,SYSTYPE=""""""VMS"""""")
$ CC_PREF = CC_PREF + "/DEFINE=(SIGUSR2=SIGKILL,SYSTYPE=""""""VMS"""""")
$ CC_PREF = CC_PREF + "/INCLUDE=(''includes')"
$ CC_PREF = CC_PREF + "/OBJECT=''OBJDIR'"
$ if (P2 .nes. "")
$   then
$ SET VER
$ CC'CC_PREF' 'P2'
$ SET NOVER
$   else
$ SET VER
$ CC'CC_PREF' OS_vms
$ CC'CC_PREF' addrbook
$ CC'CC_PREF' adrbklib
$ CC'CC_PREF' args
$ CC'CC_PREF' context
$ CC'CC_PREF' date
$ CC'CC_PREF' filter
$ CC'CC_PREF' folder
$ CC'CC_PREF' help
$ CC'CC_PREF' helptext
$ CC'CC_PREF' imap
$ CC'CC_PREF' init
$ CC'CC_PREF' mailcap
$ CC'CC_PREF' mailcmd
$ CC'CC_PREF' mailindx
$ CC'CC_PREF' mailpart
$ CC'CC_PREF' mailview
$ CC'CC_PREF' newmail
$ CC'CC_PREF' other
$ CC'CC_PREF' pine
$ CC'CC_PREF' print
$ CC'CC_PREF' reply
$ CC'CC_PREF' screen
$ CC'CC_PREF' send
$ CC'CC_PREF' signals
$ CC'CC_PREF' status
$ CC'CC_PREF' strings
$ CC'CC_PREF' tty_vms
$ SET NOVER
$ endif
$ write sys$output "%VMSBUILD-I-CRELIBR, Creating PINE.OLB library"
$ LIBRARY/OBJECT/CREATE/INSERT 'OBJDIR'PINE 'OBJDIR'*.obj
$ PURGE/NOLOG 'OBJDIR'PINE.OLB
$ PURGE/NOLOG 'OBJDIR'*.OBJ
$
$!
$! ----------------------------------------------------------------------------
$! Link the executable
$! ----------------------------------------------------------------------------
$LINK:
$ if COMPILEONLY then $ exit
$ write sys$output "%VMSBUILD-I-LINK,  Linking PINE with ", NETSTACK, " TCP support"
$ IF (COMPILER .EQS. "DEC C")
$ THEN
$	DEFINE/USER OBJ 'CCLOBJDIR'
$	LINK/EXE='EXEDIR' 'OBJDIR'pine,-
		'OBJDIR'pine/libr,-
		'OBJDIR'c-client/libr,-
		'OBJDIR'pico/libr'LINK_OPT'
$ ELSE
$	DEFINE/USER OBJ 'CCLOBJDIR'
$	LINK/EXE='EXEDIR' 'OBJDIR'pine, -
	'OBJDIR'addrbook,adrbklib,args,context,date,filter,folder, -
	help,helptext,imap,init,mailcap,mailcmd,mailindx,mailview,mailpart,newmail, -
	status,other,print,reply,screen,send,signals,strings,tty_vms,os_vms, -
	'OBJDIR'pico/libr,-
        'CCLOBJDIR'imap2,mail,misc,nntp,nntpcvms,newsrc,os_vms,rfc822,smtp,-
        sm_vms,vms_mail'LINK_OPT'
$ ENDIF
$ PURGE/NOLOG 'EXEDIR'*.EXE
