/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

Under unix, utilities started via the inetd daemon can simply read/write
the stdin/stdout/stderr files and have the I/O use the network. That is, the
utility need know nothing about the network.

Under VMS however, while the utility can be started via a 'master server' (E.G 
MULTINET_SERVER if running MULTINET TCP/IP), the channels/sockets so returned
are not necessarily accessible via stdin/stdout/stderr.  Socket descriptors and
file descriptors MUST be kept separate thus functions like printf cannot be
used directly to the network.

This code tries to kludge things by diverting the standard I/O calls into 
private routines that distinguish between network and file i/o. This is not
100% successful, there are limitations, but the code is sufficient for many
simple tasks.

The first section consists of the necessary declarations and MUST be placed
before the code which uses the I/O functions. The second section provides the
intercept routines and special io routines and should be placed at the end.

It is necessary to define a symbol as follows:

   /define=VMSIO		Use the special I/O redirect code

If VMSIO is not defined, the program uses no special network code, just like on
unix.

						Andy Harper, August 1996
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/


/*  Special definitions to re-route stdio calls to the private I/O routines */
static void __openio();
static int    special_getchar();
static char  *special_fgets();
static int    special_putchar();
static int    special_puts();
static int    special_fputs();
static int    special_printf(char *fmt,...);
static int    special_fwrite();
static int    special_fflush();

#ifdef  getchar				/* In case implemented as a macro */
#undef  getchar
#endif
#define getchar special_getchar
#define fgets   special_fgets
#define putchar special_putchar
#define puts    special_puts
#define fputs   special_fputs
#define printf  special_printf
#define fwrite  special_fwrite
#define fflush  special_fflush


/* ------ END OF VMSIO DECLARATIONS ------ */
