$! -----------------
$! Set up the logical names necessary to run the IMAPD daemon
$! Each should be tailored to the local system requirements.
$!
$! In this code, we assume that a root directory has been set up for IMAPD
$! having the following structure:
$!  [000000]    Root directory, contains this command procedure
$!  [BIN_VAX]   Executables for the VAX architecture
$!  [BIN_ALPHA] Executables for the ALPHA architecture
$!  [CONFIG]    Common configuration directory
$!  [LOGS]	Logfile directory
$!
$!
$!	See Also the CCLIENT_SETUP.COM module, for logical names that
$!	affect the daemon indirectly.
$!						Andy Harper, August 1996
$! -----------------
$!
$! -----------------
$! Root and configuration directory
$! -----------------
$ node = f$getsyi("NODENAME")
$ defsys = "define /system /exec"
$ thisdir = f$parse("Z.Z;",f$environment("PROCEDURE"),,,"NO_CONCEAL") - "Z.Z;"
$ rootdir = thisdir - "][" - "]" - "000000" + "]" 
$ if f$extract(f$length(rootdir)-2,-1,rootdir) .nes. ".]" then -
  rootdir = rootdir - "]" + ".]" 
$ defsys /tran=conc	imapd_root	'rootdir			! Root
$ defsys		imapd_conf_dir	imapd_root:[config]		! Config
$
$! -----------------
$! location of architecture specific image(s)
$! -----------------
$ Arch = "VAX"
$ if f$getsyi("VERSION") .ges. "V5.5" then $ Arch = f$getsyi("ARCH_NAME")
$ defsys		imapd_dir	imapd_root:[bin_'arch']	! Executables
$ defsys		imapd_exe	imapd_dir:imapd.exe	! The daemon
$
$
$! -----------------
$! Logging -- define these logicals to get a logfile for each IMAPD activation
$!
$!	IMAPD_LOGFILE defines where the logging goes
$!	IMAPD_LOGLEVEL defines the level of detail 0=none, 10 = highest
$!
$!	Either not defined, or the file inaccessible, gives no logging.
$!
$! -----------------
$!defsys		imapd_logfile	imapd_root:[logs]imapd-'node'.log
$!defsys		imapd_loglevel  2
$
$
$! -----------------
$! Alerts  -- if this logical is defined to a readable text file, the contents
$!		of that file are sent to the IMAP client when it is started and
$!		at any time that the contents are changed.
$!		Each line of the file is sent to the client, in this format:
$!		   * OK [ALERT] text
$!		It is the clients responsibility to detect and display this.
$! -----------------
$ defsys		imapd_alertfile imapd_root:[config]alerts.txt	! Announcements
$
$
$! -----------------
$! Idle timeout -- define this logical to force an idle timeout after 'nn'
$!                 seconds. If not defined, there is no timeout.
$! -----------------
$ defsys		imapd_timeout	1800
$
$! -----------------
$! Login failures -- define this logical to the maximum number of login
$!		     failures the server can accept before disconnecting
$! -----------------
$ defsys		imapd_logtry	3
