$! Procedure:
$!	Set up the underlying TCP/IP network stack software to initiate
$!	a network service.
$!
$! Parameters:
$!	P1	Network stack (UCX, MULTINET)
$!	P2	Service name
$!	P3	Port Number
$!
$! Author:
$!	Andy Harper	1.0	Nov 1996	Original Version
$!	Andy Harper	1.1	Dec 1996	Fixed a few problems
$!	Andy Harper	1.2	Jan 1997	Add a rudimentary CMU setup
$!	Andy Harper	1.3	Jan 1997	Add a rudimentary TCPWARE setup
$!	Andy Harper	1.4	Jan 1997	Add checks for stack presence
$!
$! Acknowledgements:
$!	Ian Miller at Softel for the UCX set up information	
$!	Curtis Williams at Western Kentucky University for the TCPWARE info
$
$
$ set noon
$
$! Check parameter
$ if (P1 .eqs. "")
$   then
$     write sys$error "You must specify the network stack name as P1"
$     exit
$ endif
$
$ if (P2 .eqs. "")
$   then
$     write sys$error "You must specify the service name name as P2"
$     exit
$ endif
$
$ if (P3 .eqs. "")
$   then
$     write sys$error "You must specify the service port name as P3"
$     exit
$ endif
$
$
$! Fixed configuration
$ TEMPFILE = "SYS$SCRATCH:SERVICE-SETUP" + f$getjpi("","PID") + ".TMP"
$ PARAMALIGN = 10
$ PARAMPREFIX= "STACK"
$
$! Parameters that may be changed by the user (not all are used by all stacks)
$ STACK$SERVICE   = P2
$ STACK$PORT      = P3
$ STACK$TYPE      = "TCP"
$ STACK$IMAGE     = "''STACK$SERVICE'_EXE"
$ STACK$USERNAME  = "SYSTEM"
$ STACK$UIC       = "[1,3]"
$ STACK$PRCNAM    = P2
$ STACK$PROCEDURE = "''STACK$SERVICE'_ROOT:[000000]''STACK$SERVICE'.COM"
$ STACK$STARTLOG  = "''STACK$SERVICE'_ROOT:[000000]''STACK$SERVICE'_STARTUP.LOG"
$ STACK$MAXSESS   = "20"
$ STACK$PRIVS     = "TMPMBX,NETMBX,SYSPRV"
$ STACK$PGFLQUO   = f$string( f$getsyi("PQL_DPGFLQUOTA") )
$ STACK$MAXBUF    = "8300"
$
$
$! If UCX specified, set up the stack
$ if (P1 .eqs. "UCX")
$   then
$
$     if f$trnlnm("UCX$INET_HOST","LNM$SYSTEM") .eqs. ""
$       then
$         write sys$output "%UCX-E-NOTFOUND, UCX not present/loaded on this node"
$         exit
$     endif
$
$   ! Define UCX specific params
$     STACK$PRCNAM  = "UCX$" + STACK$PRCNAM
$
$   ! Check the parameters with the user
$     PARAMLIST = "SERVICE,PORT,IMAGE,PROCEDURE,USERNAME,PRCNAM,MAXSESS,STARTLOG"
$     gosub PARAMCHECK
$
$   ! Generate command procedure to configure service
$     gosub INITCONFIGPROC
$     gosub WRITECONFIGMESS
$     write CONFIGPROC "$ ucx set service ", STACK$SERVICE, " -"
$     write CONFIGPROC "   /user=", STACK$USERNAME, " -"
$     write CONFIGPROC "   /proc=", STACK$PRCNAM,   " -"
$     write CONFIGPROC "   /limit=", STACK$MAXSESS, " -"
$     write CONFIGPROC "   /port=", STACK$PORT,     " -"
$     write CONFIGPROC "   /file=", STACK$PROCEDURE," -"
$     write CONFIGPROC "   /log=(file=", STACK$STARTLOG, ", ALL)"
$     gosub WRITEENABLEMESS
$     write CONFIGPROC "$ ucx set config enable service ", STACK$SERVICE
$     write CONFIGPROC "$ ucx enable service ", STACK$SERVICE
$     gosub WRITECREPROCMESS
$     write CONFIGPROC "$ create ", STACK$PROCEDURE
$     write CONFIGPROC "$deck"
$     write CONFIGPROC "$ run ", STACK$IMAGE
$     write CONFIGPROC "$eod"
$     gosub WRITETERMMESS
$     gosub TERMCONFIGPROC
$
$    ! execute the configuration procedure
$     gosub EXECCONFIGPROC
$     gosub DELECONFIGPROC
$     exit
$ endif
$
$
$! Set up MULTINET stack
$ if (P1 .eqs. "MULTINET")
$   then
$
$     if f$trnlnm("MULTINET","LNM$SYSTEM") .eqs. ""
$       then
$         write sys$output "%MULTINET-E-NOTFOUND, MULTINET not present/loaded on this node"
$         exit
$     endif
$
$   ! Check the parameters with the user
$     PARAMLIST = "SERVICE,PORT,TYPE,IMAGE,USERNAME,PGFLQUO"
$     gosub PARAMCHECK
$
$   ! generate configuration procedure
$     gosub INITCONFIGPROC
$     gosub WRITECONFIGMESS
$     write CONFIGPROC "$ multinet configure/server"
$     write CONFIGPROC "  add ", STACK$SERVICE
$     write CONFIGPROC "  ", STACK$TYPE
$     write CONFIGPROC "  ", STACK$PORT
$     write CONFIGPROC "  ", STACK$IMAGE
$     write CONFIGPROC "  set username ", STACK$USERNAME
$     write CONFIGPROC "  set pql-pgflquo ", STACK$PGFLQUO
$     write CONFIGPROC "  disable ", STACK$SERVICE
$     write CONFIGPROC "  save"
$     gosub WRITEENABLEMESS
$     write CONFIGPROC "$ multinet configure/server"
$     write CONFIGPROC "  enable ", STACK$SERVICE
$     write CONFIGPROC "  save"
$     write CONFIGPROC "  restart"
$     gosub WRITETERMMESS
$     gosub TERMCONFIGPROC
$     gosub EXECCONFIGPROC
$     gosub DELECONFIGPROC
$     exit
$ endif
$
$
$
$! Set up TCPWARE stack
$ if (P1 .eqs. "TCPWARE")
$   then
$
$     if f$trnlnm("TCPWARE","LNM$SYSTEM") .eqs. ""
$       then
$         write sys$output "%TCPWARE-E-NOTFOUND, TCPWARE not present/loaded on this node"
$         exit
$     endif
$
$   ! Check the parameters with the user
$     PARAMLIST = "SERVICE,PORT,TYPE,IMAGE,UIC,PGFLQUO"
$     gosub PARAMCHECK
$
$   ! Check whether the relevant WKS entry appears in the services file
$   !  (uncommented..)
$     write sys$output ""
$     write sys$output P1,": Checking TCPWARE:SERVICES. file for service entry"
$     if f$search("TCPWARE:SERVICES.") .eqs. ""
$       then
$         write sys$output " Configuration file TCPWARE:SERVICES. not found"
$     endif
$
$     define/user sys$error nl:
$     define/user sys$output nl:
$     search/output='TEMPFILE' TCPWARE:SERVICES. 'STACK$PORT'
$     if f$file_attributes("''TEMPFILE'", "EOF")
$       then
$         write sys$output " An entry for this service appears in TCPWARE:SERVICES."
$         write sys$output " and looks like:"
$         write sys$output ""
$         type/nopage 'TEMPFILE'
$       else
$         write sys$output " No entry for ''STACK$SERVICE' on ''STACK$PORT' found in TCPWARE:SERVICES."
$         write sys$output " Edit this file to add the following line and then rerun SERVICE_SETUP:"
$         write sys$output ""
$         write sys$output "  ''STACK$SERVICE'	''STACK$PORT'/TCP"
$         delete/nolog 'TEMPFILE';*
$         exit
$     endif
$     write sys$output ""
$
$   ! generate configuration procedure
$     gosub INITCONFIGPROC
$     gosub WRITECONFIGMESS
$     write CONFIGPROC "$ netcu 
$     write CONFIGPROC "  add service ", STACK$PORT, " -"
$     write CONFIGPROC "  ", STACK$TYPE, " -"
$     write CONFIGPROC "  ", STACK$IMAGE, " -"
$     write CONFIGPROC "  /uic=", STACK$UIC, " -"
$     write CONFIGPROC "  /page_file=", STACK$PGFLQUO
$     gosub WRITEENABLEMESS
$     gosub TERMCONFIGPROC
$     gosub EXECCONFIGPROC
$     gosub DELECONFIGPROC
$     exit
$ endif
$
$ if (P1 .eqs. "CMU")
$   then
$
$     if f$trnlnm("CMUIP_ROOT","LNM$SYSTEM") .eqs. ""
$       then
$         write sys$output "%CMU-E-NOTFOUND, CMU not present/loaded on this node"
$         exit
$     endif
$
$   ! Extra privs required
$     STACK$PRIVS = STACK$PRIVS + ",PHY_IO"
$
$   ! Check user modifiable parameters for this stack with the user
$     PARAMLIST   = "SERVICE,PORT,IMAGE,PRIVS"
$     gosub PARAMCHECK
$
$   ! Check whether the relevant WKS entry appears in the configuration file
$   !  (uncommented..)
$     write sys$output ""
$     write sys$output P1,": Checking INET$CONFIG file for service entry"
$     if f$search("INET$CONFIG") .eqs. ""
$       then
$         write sys$output " Configuration file INET$CONFIG not found"
$     endif
$
$     define/user sys$error nl:
$     define/user sys$output nl:
$     search/output='TEMPFILE' inet$config WKS:'STACK$PORT':
$     if $status
$       then
$         write sys$output " An entry for this service appears in INET$CONFIG"
$         write sys$output " and looks like:"
$         write sys$output ""
$         type/nopage 'TEMPFILE'
$       else
$         write sys$output " No entry for ''STACK$SERVICE' on ''STACK$PORT' found in INET$CONFIG"
$         write sys$output " Edit this file to add the following line:"
$         write sys$output ""
$         write sys$output "  WKS:''STACK$PORT':''STACK$SERVICE':''STACK$IMAGE':NETWRK:''STACK$PRIVS':::::4:5"
$     endif
$     write sys$output ""
$
$     write sys$output P1,": Checking SYSGEN parameters"
$
$   ! Check that MAXBUF is sufficient
$     if f$getsyi("MAXBUF") .ge. 'STACK$MAXBUF'
$       then
$         write sys$output " -- MAXBUF is OK"
$       else
$         write sys$output " -- MAXBUF too low. Should be at least ", STACK$MAXBUF
$     endif
$
$     gosub DELECONFIGPROC
$     exit
$ endif
$
$
$! A stack we know nothing about....
$ write sys$error "''P1': Service setup not supported for this stack type"
$ exit
$
$! ------------------------------------------------------------------------
$! ------------------------------------------------------------------------
$INITCONFIGPROC:
$ write sys$output "''P1': Creating configuration procedure for ''STACK$SERVICE' on port ''STACK$PORT'"
$ open/write/error=NOOPEN CONFIGPROC 'TEMPFILE'
$ return
$NOOPEN:
$ write sys$error "Cannot open temporary file ", TEMPFILE
$ exit
$
$! ------------------------------------------------------------------------
$! ------------------------------------------------------------------------
$WRITECONFIGMESS:
$ write CONFIGPROC "$ write SYS$OUTPUT ""''P1': Configuring service ''STACK$SERVICE' on port ''STACK$PORT'"""
$ return
$
$! ------------------------------------------------------------------------
$! ------------------------------------------------------------------------
$WRITEENABLEMESS:
$ write CONFIGPROC "$ write SYS$OUTPUT ""''P1': Enabling service ''STACK$SERVICE'"""
$ return
$
$! ------------------------------------------------------------------------
$! ------------------------------------------------------------------------
$WRITECREPROCMESS:
$ write CONFIGPROC "$ write SYS$OUTPUT ""''P1': Creating procedure ''STACK$PROCEDURE'"""
$ return
$
$! ------------------------------------------------------------------------
$! ------------------------------------------------------------------------
$WRITETERMMESS:
$ write CONFIGPROC "$ write SYS$OUTPUT ""''P1': Configuration complete"""
$ return
$
$
$! ------------------------------------------------------------------------
$! ------------------------------------------------------------------------
$TERMCONFIGPROC:
$ close CONFIGPROC
$ return
$
$! ------------------------------------------------------------------------
$! Execute the configuration procedure and check error return
$! ------------------------------------------------------------------------
$EXECCONFIGPROC:
$ write sys$output "''P1': Executing configuration procedure now..."
$
$! type/nopage 'TEMPFILE'
$! return
$
$ @'TEMPFILE'
$ if .not. $status
$   then
$     write sys$error "''P1': config procedure failed. Status = ''$status', Message=", f$message($status)
$ endif
$ return
$
$! ------------------------------------------------------------------------
$! Delete the temporary configuration procedure
$! ------------------------------------------------------------------------
$DELECONFIGPROC:
$ delete 'TEMPFILE';*
$ return
$
$!
$! ------------------------------------------------------------------------
$! Scan list of parameters and confirm the value of each with the user
$!
$! Inputs:
$!	PARAMALIGN	Indentation of prompt
$!	PARAMPREFIX	Prefix used to form symbol name from parameter name
$!	PARAMLIST	List of parameter names
$!      'PREFIX'$'name'	List of symbols containing values of parameter 'name'
$!
$! Outputs:
$!      'PREFIX'$'name'	List of symbols containing values of parameter 'name'
$!
$! ------------------------------------------------------------------------
$PARAMCHECK:
$   write sys$output ""
$   write sys$output "Please confirm, or modify, the following parameters:"
$   write sys$output ""
$   N = 0
$  GUIloop:
$   PARAMID = f$element(N,",",PARAMLIST)
$   if (PARAMID .nes. ",")
$     then
$        PARAMVALUE = 'PARAMPREFIX'$'PARAMID'
$        gosub ConfirmValue
$        'PARAMPREFIX'$'PARAMID' = PARAMVALUE
$        N = N+1
$        goto GUIloop
$   endif
$
$   write sys$output ""
$   read/prompt="  *  OK to proceed [N] ? " sys$command PARAMVALUE
$   if .not. PARAMVALUE then $ goto PARAMCHECK
$ return
$ 
$! ------------------------------------------------------------------------
$! Display the parameter, if user specifies new value, replace it
$!
$! Inputs:
$!	PARAMID		Name of parameter to be confirmed
$!	PARAMVALUE	Current value of parameter
$!	PARAMALIGN	Indentation depth, for alignment
$!
$! Outputs:
$!	PARAMVALUE	New value of parameter
$!
$! ------------------------------------------------------------------------
$ConfirmValue:
$   PromptString = f$fao("  * !#* !AS : [!AS] ?", PARAMALIGN-f$length(PARAMID), PARAMID, PARAMVALUE)
$   read/prompt=&PromptString sys$command PromptResponse
$   if PromptResponse .nes. "" then $ PARAMVALUE = PromptResponse
$ return
