$! Purpose:
$!	Create a new user account suitable for running a network server
$!
$! Parameters:
$!	P1	Username
$!	P2	UIC
$!	P3	Network service which will run under this username
$!
$! Author:
$!	1.0	Andy Harper	7-NOV-1996	Original Version
$!	1.1	Andy Harper	2-JAN-1997	Deal with location of SYSUAF properly
$!
$! Acknowledgements:
$!	Ian Miller of softel for the basic idea
$
$
$ AUTH = "call AUTHORIZE"
$
$! Check mandatory parameters
$ if (P1 .eqs. "") .or. (P2 .eqs. "") .or. (P3 .eqs. "")
$   then
$     write sys$error "Must supply P1=username,P2=uic and P3=service"
$     exit
$ endif
$
$! Set up defaults
$ UAI$USERNAME   = P1
$ UAI$UIC        = P2
$ UAI$ACCOUNT    = P3
$ UAI$OWNER      = "''P3' Server"
$ UAI$DEVICE     = "''P3'_ROOT"
$ UAI$DIRECTORY = "[000000]"
$ UAI$FLAGS      = "DISCTLY,DEFCLI,RESTRICTED"
$ UAI$ACCESS     = "/NETWORK/NOBATCH/NOLOCAL/NODIALUP/NOREMOTE"
$ UAI$PRIVILEGE  = "TMPMBX,NETMBX,SYSPRV"
$ UAI$PGFLQUO    = f$string(f$getsyi("PQL_DPGFLQUOTA"))
$
$
$! Now ask user to confirm selected values ...
$ PARAMLIST   = "USERNAME,UIC,ACCOUNT,OWNER,DEVICE,DIRECTORY,FLAGS,ACCESS,PRIVILEGE,PGFLQUO"
$ PARAMALIGN  = 9
$ PARAMPREFIX = "UAI"
$ gosub PARAMCHECK
$
$ UAI$DEVICE = UAI$DEVICE - ":"
$ if f$trnlnm(UAI$DEVICE,"LNM$SYSTEM") .eqs. ""
$   then
$      write sys$error "''UAI$DEVICE' logical name must be defined to root directory"
$      exit
$ endif
$
$! Set up the account
$ write sys$output "Creating user ''UAI$USERNAME' with UIC ''UAI$UIC'"
$ AUTH add 'UAI$USERNAME'/uic='UAI$UIC'/account='UAI$ACCOUNT' "/owner=""''UAI$OWNER'"""
$ if $status
$   then
$     write sys$output "User account ''UAI$USERNAME' created, uic=''UAI$UIC'"
$   else
$     write sys$error "Unable to add user account ''UAI$USERNAME', error ''$status' (", f$message($status), ")"
$     exit
$ endif
$
$ AUTH modify 'UAI$USERNAME' /device='UAI$DEVICE'    /directory='UAI$DIRECTORY' 
$ AUTH modify 'UAI$USERNAME' 'UAI$ACCESS'            /flags=('UAI$FLAGS')
$ AUTH modify 'UAI$USERNAME' /priv=('UAI$PRIVILEGE') /defpriv=('UAI$PRIVILEGE')
$ AUTH modify 'UAI$USERNAME' /pgflquo='UAI$PGFLQUO'
$
$! Restricted accounts require a LOGIN.COM
$ write sys$output "Creating LOGIN.COM file for user ''UAI$USERNAME'"
$ create 'UAI$DEVICE':'UAI$DIRECTORY'LOGIN.COM
$DECK
$! LOGIN.COM for IMAPD
$EOD
$ exit
$
$! ------------------------------------------------------------------------
$! Scan list of parameters and confirm the value of each with the user
$!
$! Inputs:
$!	PARAMALIGN	Indentation of prompt
$!	PARAMPREFIX	Prefix used to form symbol name from parameter name
$!	PARAMLIST	List of paramter names
$!      'PREFIX'$'name'	List of symbols containing values of parameter 'name'
$!
$! Outputs:
$!      'PREFIX'$'name'	List of symbols containing values of parameter 'name'
$!
$! ------------------------------------------------------------------------
$PARAMCHECK:
$   write sys$output ""
$   write sys$output "Please confirm, or modify, the following parameters:"
$   write sys$output ""
$   N = 0
$  GUIloop:
$   PARAMID = f$element(N,",",PARAMLIST)
$   if (PARAMID .nes. ",")
$     then
$        PARAMVALUE = 'PARAMPREFIX'$'PARAMID'
$        gosub ConfirmValue
$        'PARAMPREFIX'$'PARAMID' = PARAMVALUE
$        N = N+1
$        goto GUIloop
$   endif
$
$   write sys$output ""
$   read/prompt="  *  OK to proceed [N] ? " sys$command PARAMVALUE
$   if .not. PARAMVALUE then $ goto PARAMCHECK
$ return
$ 
$! ------------------------------------------------------------------------
$! Display the parameter, if user specifies new value, replace it
$!
$! Inputs:
$!	PARAMID		Name of parameter to be confirmed
$!	PARAMVALUE	Current value of parameter
$!	PARAMALIGN	Indentation depth, for alignment
$!
$! Outputs:
$!	PARAMVALUE	New value of parameter
$!
$! ------------------------------------------------------------------------
$ConfirmValue:
$   PromptString = f$fao("  * !#* !AS : [!AS] ?", PARAMALIGN-f$length(PARAMID), PARAMID, PARAMVALUE)
$   read/prompt=&PromptString sys$command PromptResponse
$   if PromptResponse .nes. "" then $ PARAMVALUE = PromptResponse
$ return
$
$! ------------------------------------------------------------------------
$! AUTHORIZE - modify the system UAF taking into account any logicals defining
$! where the SYSUAF resides.
$! ------------------------------------------------------------------------
$AUTHORIZE: SUBROUTINE	! P1=command, P2=username P3-P8=parameters
$ if f$trnlnm("SYSUAF")     .eqs. "" then $ define/user sysuaf     sys$system:sysuaf.dat
$ if f$trnlnm("RIGHTSLIST") .eqs. "" then $ define/user rightslist sys$system:rightslist.dat
$ mc authorize 'P1' 'P2' 'P3' 'P4' 'P5' 'P6' 'P7' 'P8'
$ENDSUBROUTINE
